unit u_Speech;

interface

uses
  SysUtils;

const
  SVSFDefault = 0;
  SVSFlagsAsync = 1;
  SVSFPurgeBeforeSpeak = 2;
  SVSFIsFilename = 4;
  SVSFIsXML = 8;
  SVSFPersistXML = 32; //$00000020;
  SVSFParseSapi = 128; //$00000080;
  SVSFParseSsml = 256; //$00000100;
  SVSFNLPMask = $00000040;
  SVSFParseMask = $00000180;

procedure Speak(const AText: string; const Flags: Longword = SVSFDefault);
procedure SpeakPitch(const AText: string; const AValue: Integer = 0; const Flags: Longword = 0); // -10..10
procedure Rate(AValue: Integer); // -10..10
procedure Volume(AValue: Integer); // 0..100
procedure RoughSpeechExceptionHandler(Self, Sender: TObject; E: Exception);

implementation

uses
  Variants, ComObj, Windows;

resourcestring
//  StrSAPISpVoice = 'SAPI.SpVoice';
  StrSAPISpVoice = 'SPEECH.SpVoice';
var
  gVoice: Variant;

procedure Speak(const AText: string; const Flags: Longword = 0);
const
  // PRON SYM does not work with SPEECH...
  Sssh = '<PRON SYM="s s !"/><silence msec="500"/>'; //'<PRON SYM="s ">s!</PRON>';
begin
  if VarIsClear(gVoice) then
    gVoice := CreateOLEObject(StrSAPISpVoice);
  if AText = '' then
    gVoice.Speak(Sssh, SVSFlagsAsync or SVSFIsXML) // Exit
  else
  begin
    gVoice.Speak(AText, {SVSFlagsAsync or} Flags);
  end;
end;

procedure SpeakPitch(const AText: string; const AValue: Integer = 0; const Flags: Longword = 0);
const
  sPitch = '<pitch absmiddle="%d"/>';
begin
  if VarIsClear(gVoice) then
    gVoice := CreateOLEObject(StrSAPISpVoice);
  gVoice.Speak(Format(sPitch, [AValue]) + AText, SVSFPersistXML or SVSFIsXML or {SVSFlagsAsync or} Flags);
end;

procedure Rate(AValue: Integer);
begin
  if VarIsClear(gVoice) then
    gVoice := CreateOLEObject(StrSAPISpVoice);
  gVoice.Rate := AValue;
end;

procedure Volume(AValue: Integer);
begin
  if VarIsClear(gVoice) then
    gVoice := CreateOLEObject(StrSAPISpVoice);
  gVoice.Volume := AValue;
end;

procedure SafeVistaSpeak(const AVoice: Variant; const AText: string; const AFlags:Integer = 0);
var
  SavedCW: Word;
begin
  SavedCW := Get8087CW;
  // problem analog to calls to OpenGL. see Help for Set8087CW
  // $4 = zero divide bit. see http://blog.neslekkim.net/2008/10/fpu-issues-when-interoping-delphi-and.html
  Set8087CW(SavedCW or $4);
  AVoice.Speak(AText, SVSFDefault);
  Set8087CW(SavedCW);
end;

procedure RoughSpeechExceptionHandler(Self, Sender: TObject; E: Exception);
begin
  try
    Speak(E.Message, SVSFPurgeBeforeSpeak);
  except
    // Don't want any (Speech) exception during exception handling
  end;
  ShowException(ExceptObject, ExceptAddr);
end;

{
<rate absspeed="1"/><pitch absmiddle="-7"/>
Lily, come here to help me!
<rate absspeed="3"/><pitch absmiddle="+10"/>
<emph><partofsp part="interjection">Mom!</partofsp></emph> It's not fair! Should be David's turn!
<pitch absmiddle="-7"/><rate absspeed="1"/>
<silence msec="500"/> Please come here <emph>immediately!</emph> He's doing his homework.

  FVoice.Speak('C:\Users\fgaillard\Documents\multiplication\SOUND16.WAV', SVSFIsFilename);
  FVoice.Speak('http://www.wideorbit.com', SVSFIsFilename);

}
end.