unit USpeech;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComObj, StdCtrls;

type
  TForm4 = class(TForm)
    btnForm: TButton;
    btnLocal: TButton;
    Memo1: TMemo;
    chkAsynchrone: TCheckBox;
    chkNonStop: TCheckBox;
    procedure btnFormClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnLocalClick(Sender: TObject);
    procedure Memo1DblClick(Sender: TObject);
  private
    { Private declarations }
    FVoice: Variant;
  public
    { Public declarations }
  end;

var
  Form4: TForm4;

implementation

resourcestring
//  StrSAPISpVoice = 'SAPI.SpVoice';
  StrSAPISpVoice = 'SPEECH.SpVoice';

{$R *.dfm}

const
  SVSFDefault = 0;
  SVSFIsXML = 8;
  SVSFlagsAsync = 1;

procedure TForm4.btnFormClick(Sender: TObject);
begin
  FVoice.speak(Memo1.Text, SVSFlagsAsync);
end;

procedure TForm4.btnLocalClick(Sender: TObject);
var
  Voice: Variant;
  SavedCW: Word;
  Synchrone: Boolean;
begin
  Voice := CreateOLEObject(StrSAPISpVoice);
  Synchrone := not chkAsynchrone.Checked;
  if Synchrone then
  begin
  // default = sync. Poses problems on Vista/2008/7 with SAPI (not with SPEECH)
    SavedCW := Get8087CW;
    // problem analog to calls to OpenGL. see Help for Set8087CW
    // $4 = zero divide bit. see http://blog.neslekkim.net/2008/10/fpu-issues-when-interoping-delphi-and.html
    Set8087CW(SavedCW or $4);
    Voice.speak(Memo1.Text, SVSFDefault);
    Set8087CW(SavedCW);
  end
  else
  begin
  // async
    Voice.speak(Memo1.Text, SVSFlagsAsync);
  // but with a local interface variable, you need to wait until it's finished
  // or it's deleted as soon as out of scope, i.e. before actually speaking
    if chkNonStop.Checked then
      while not Voice.WaitUntilDone(100) do // 100ms
        Application.ProcessMessages
    else
      Voice.WaitUntilDone(-1); // -1 = INFINITE
  // or
  //  WaitForSingleObject(Voice.SpeakCompleteEvent, INFINITE);
  end;
end;

procedure TForm4.FormCreate(Sender: TObject);
begin
  FVoice := CreateOLEObject(StrSAPISpVoice);
  // SPEECH treats CRLF like '.' and drops the voice to end the sentence.
  // Be sure to remove the CRLF from Memo.Text with Speech Server TTS
  Memo1.Text := 'La reconnaissance vocale ou reconnaissance automatique de la '+
    'parole est une technique informatique qui permet d''analyser un mot ou '+
    'une phrase capte au moyen d''un microphone pour la transcrire sous '+
    'la forme d''un texte exploitable par une machine.';
end;


type THackControl = class(TControl);
procedure TForm4.Memo1DblClick(Sender: TObject);
begin
  // Dbl Click on the Memo for SSML interpretation
  FVoice.Speak(THackControl(Sender as TControl).Text, SVSFIsXML);
end;

end.
{
La reconnaissance vocale ou reconnaissance automatique de la parole (Automatic Speech Recognition ASR) est une technique informatique qui permet d'analyser un mot ou une phrase capte au moyen d'un microphone pour la transcrire sous la forme d'un texte exploitable par une machine.
}
{
Speech synthesis is the artificial production of humanspeech.
A computer system used for this purpose is called a speech synthesizer, and can be implemented in software, or hardware.
}
